package org.rcisoft.business.bslschedule.service.impl;

import com.alibaba.fastjson.JSON;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.rcisoft.business.barrange.dao.BArrangeRepository;
import org.rcisoft.business.barrange.entity.ScheduleDto;
import org.rcisoft.business.bcoursecode.dao.BCourseCodeRepository;
import org.rcisoft.business.bcoursecode.entity.BCourseCode;
import org.rcisoft.business.beduclass.dao.BEduClassRepository;
import org.rcisoft.business.beduclass.entity.BEduClass;
import org.rcisoft.business.blesson.dao.BLessonRepository;
import org.rcisoft.business.blesson.entity.BLesson;
import org.rcisoft.business.brooms.dao.BRoomsRepository;
import org.rcisoft.business.brooms.entity.BRooms;
import org.rcisoft.business.bsl.entity.BSl;
import org.rcisoft.business.bslschedule.entity.ScheduleResult;
import org.rcisoft.business.bslschedule.util.FindTime;
import org.rcisoft.business.bterm.dao.BTermRepository;
import org.rcisoft.business.bterm.entity.BTerm;
import org.rcisoft.business.bterm.utils.Week;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.service.RcRedisService;
import org.rcisoft.core.service.SerializationUtils;
import org.rcisoft.core.util.ExcelUtil;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.business.bslschedule.dao.BSlScheduleRepository;
import org.rcisoft.business.bslschedule.entity.BSlSchedule;
import org.rcisoft.business.bslschedule.service.BSlScheduleService;


import org.rcisoft.sys.user.dao.SysUserMapper;
import org.rcisoft.sys.user.entity.SysUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;


import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

import lombok.extern.slf4j.Slf4j;

/**
 * Created by  on 2018-5-2 16:39:34.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class BSlScheduleServiceImpl implements BSlScheduleService {

    @Autowired
    private BSlScheduleRepository bSlScheduleRepository;

    @Autowired
    private BTermRepository bTermRepository;

    @Autowired
    private BCourseCodeRepository bCourseCodeRepository;

    @Autowired
    private BArrangeRepository bArrangeRepository;

    @Autowired
    private BLessonRepository bLessonRepository;

    @Autowired
    private SysUserMapper sysUserMapper;

    @Autowired
    private BRoomsRepository bRoomsRepository;

    @Autowired
    private BEduClassRepository bEduClassRepository;

    @Autowired
    private RcRedisService rcRedisServiceImpl;


    private static Map<String,Date> timeMap;

    private static String termCode;

    /**
    * 保存 bSlSchedule
    * @param bSlSchedule
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel save(BSlSchedule bSlSchedule){
        bSlSchedule.setCommonBusinessId();
        //增加操作
        UserUtil.setCurrentPersistOperation(bSlSchedule);
        int line = bSlScheduleRepository.insertSelective(bSlSchedule);
        log.info(UserUtil.getUserInfoProp(bSlSchedule.getToken(),UserUtil.USER_USERNAME)+"新增了ID为"+
            bSlSchedule.getBusinessId()+"的信息");
        return new PersistModel(line);
    }

    /**
    * 逻辑删除
    * @param bSlSchedule
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel remove(BSlSchedule bSlSchedule){
        UserUtil.setCurrentMergeOperation(bSlSchedule);
        bSlSchedule.setDeleted();
        int line = bSlScheduleRepository.logicalDelete(bSlSchedule);
        log.info(UserUtil.getUserInfoProp(bSlSchedule.getToken(),UserUtil.USER_USERNAME)+"逻辑删除了ID为"+
        bSlSchedule.getBusinessId()+"的信息");
        return new PersistModel(line);
    }

    /**
    * 修改 bSlSchedule
    * @param bSlSchedule
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel merge(BSlSchedule bSlSchedule){
        UserUtil.setCurrentMergeOperation(bSlSchedule);
        int line = bSlScheduleRepository.updateByPrimaryKeySelective(bSlSchedule);
        log.info(UserUtil.getUserInfoProp(bSlSchedule.getToken(),UserUtil.USER_USERNAME)+"修改了ID为"+
            bSlSchedule.getBusinessId()+"的信息");
        return new PersistModel(line);
    }

    /**
    * 根据id查询 bSlSchedule
    * @param id
    * @return
    */
    public BSlSchedule findById(String id){
        return bSlScheduleRepository.selectByPrimaryKey(id);
    }

    /**
    * 分页查询 bSlSchedule
    * @param bSlSchedule
    * @return
    */
    public List<BSlSchedule> findAllByPagination(PageUtil<BSlSchedule> paginationUtility,
        BSlSchedule bSlSchedule){
        bSlSchedule.setStart();
        bSlSchedule.setNotDeleted();
        return bSlScheduleRepository.queryBSlSchedules(bSlSchedule);
    }

    /**
     * 按周与学期编号查询 bSlSchedule
     * @param week,termCode1,subAgencyId
     * @return
     */
    @Override
    public ScheduleResult queryMoreBSlSchedules(String week,String termCode1 , String subAgencyId) {
        ScheduleResult scheduleResult = new ScheduleResult();
        BTerm bTerm = bTermRepository.queryBtermByCode(termCode1).get(0);
        if(bTerm.getWeek() == null || bTerm.getWeek().equals("")){
            scheduleResult.setWeekNum(String.valueOf(Week.buildList(bTerm.getStartDate(),bTerm.getEndDate()).size() + 1));
        }else{
            scheduleResult.setWeekNum(bTerm.getWeek());
        }
        Map<String,Date> map = FindTime.findTime(bTerm.getStartDate(),bTerm.getEndDate());
        Date min = new Date();
        //按周获取时间段
        if(map.get(week+",1") != null && !map.get(week+",1").toString().equals("")){
            min = map.get(week+",1");
        }else if (map.get(week+",2") != null && !map.get(week+",2").toString().equals("")){
            min = map.get(week+",2");
        }else if (map.get(week+",3") != null && !map.get(week+",3").toString().equals("")){
            min = map.get(week+",3");
        }else if (map.get(week+",4") != null && !map.get(week+",4").toString().equals("")){
            min = map.get(week+",4");
        }else if (map.get(week+",5") != null && !map.get(week+",5").toString().equals("")){
            min = map.get(week+",5");
        }else if (map.get(week+",6") != null && !map.get(week+",6").toString().equals("")){
            min = map.get(week+",6");
        }else if (map.get(week+",7") != null && !map.get(week+",7").toString().equals("")){
            min = map.get(week+",7");
        }
        Calendar minCal = Calendar.getInstance();
        minCal.setTime(min);
        Calendar maxCal = Calendar.getInstance();
        maxCal.setTime(min);
        while(maxCal.get(Calendar.DAY_OF_WEEK) != Calendar.SUNDAY){
            maxCal.add(Calendar.DATE,1);
        }
        SimpleDateFormat format0 = new SimpleDateFormat("yyyy-MM-dd");
        List<BSlSchedule> list = bSlScheduleRepository.queryMoreBSlSchedules(
                format0.format(minCal.getTime()),
                format0.format(maxCal.getTime()),
                subAgencyId);

//        for(BSlSchedule bSlSchedule : list){
//            int minWeek = 0;
//            int maxWeek = 0;
//            //共几周课
//            int total = 0;
//            String weekDetail = "";
//            String key = bSlSchedule.getSubAgencyId() +
//                    bSlSchedule.getBEduClass() +
//                    bSlSchedule.getBLesson() + "";
//            List<ScheduleDto> scheduleDtos = rcRedisServiceImpl.getList(key);
//            //如果缓存内不存在数据
//            if(scheduleDtos == null || scheduleDtos.size() == 0){
//                List<ScheduleDto> list1 = bArrangeRepository.queryDtoThroughBsl(bSlSchedule.getSubAgencyId(),bSlSchedule.getEduClassCode(),bSlSchedule.getBLesson().getCode());
//                for(int i = 0 ; i < list1.size() ; i++){
//                    String innerKey = key + i;
//                    rcRedisServiceImpl.setBytes(innerKey, SerializationUtils.serializer(list.get(i)));
//                    //获取周
//                    String detailWeek = list1.get(i).getWeek();
//                    //分割周
//                    String[] subWeek = detailWeek.split("-");
//                    //确定这段时间有几周
//                    if(subWeek.length > 1){
//                        if(i == 0)
//                            minWeek = Integer.valueOf(subWeek[0]);
//                        maxWeek = Integer.valueOf(subWeek[1]);
//                        total += (Integer.valueOf(subWeek[1]) - Integer.valueOf(subWeek[0]));
//                        weekDetail += "(" + detailWeek + "周)";
//                    }else{
//                        if(i == 0)
//                            minWeek = Integer.valueOf(detailWeek);
//                        maxWeek = Integer.valueOf(detailWeek);
//                        total += 1;
//                        weekDetail += "(" + detailWeek + "周)";
//                    }
//                }
//                bSlSchedule.setResult("共 " + total + "周，" + weekDetail + FindTime.getStartOrEndTime(minWeek,"getStart") + FindTime.getStartOrEndTime(maxWeek,"getEnd"));
//                bSlSchedule.setList(list1);
//            }else{
//                //如果存在
//                List<ScheduleDto> list1 = new ArrayList<>();
//                for(int i = 0 ; true ; i++){
//                    byte[] bytes = rcRedisServiceImpl.getBytes(key + i);
//                    if(bytes == null || bytes.length == 0){
//                        break;
//                    }
//                    ScheduleDto scheduleDto = SerializationUtils.deserializer(bytes,ScheduleDto.class);
//                    String detailWeek = scheduleDto.getWeek();
//                    String[] subWeek = detailWeek.split("-");
//                    list1.add(scheduleDto);
//                    if(subWeek.length > 1){
//                        if(i == 0)
//                            minWeek = Integer.valueOf(subWeek[0]);
//                        maxWeek = Integer.valueOf(subWeek[1]);
//                        total += (Integer.valueOf(subWeek[1]) - Integer.valueOf(subWeek[0]));
//                        weekDetail += "(" + detailWeek + "周)";
//                    }else{
//                        if(i == 0)
//                            minWeek = Integer.valueOf(detailWeek);
//                        maxWeek = Integer.valueOf(detailWeek);
//                        total += 1;
//                        weekDetail += "(" + detailWeek + "周)";
//                    }
//                }
//                bSlSchedule.setResult("共 " + total + "周，" + weekDetail + FindTime.getStartOrEndTime(minWeek,"getStart") + FindTime.getStartOrEndTime(maxWeek,"getEnd"));
//                bSlSchedule.setList(list1);
//            }
//        }
        scheduleResult.setBSlSchedules(list);
        BCourseCode bCourseCode = new BCourseCode();
        bCourseCode.setFlag("1");
        bCourseCode.setDelFlag("0");
        scheduleResult.setBCourseCodes(bCourseCodeRepository.queryBCourseCodes(bCourseCode));
        return scheduleResult;
    }

}
